@@ -19,7 +19,7 @@ print.profile.gnm <- function (x, digits = max(3, getOption("digits") - 3), ...)
        # coefs <- coefs[-seq(attr(x$cov.scaled, "eliminate")), ]
 
     if (length(x)) {
-        if (any(sapply(x, function(x) isTRUE(is.na(x)))))
+        if (any(vapply(x, function(x) isTRUE(is.na(x)), TRUE)))
             cat("\nProfile is NA where coefficient has been constrained or",
                 "is unidentified\n\n")
         print.default(x)
