@@ -1,7 +1,30 @@
+/*  Copyright (C) 2005, 2006, 2008-2010, 2017 Heather Turner */
+/*  */
+/*  This program is free software; you can redistribute it and/or modify */
+/*  it under the terms of the GNU General Public License as published by */
+/*  the Free Software Foundation; either version 2 or 3 of the License */
+/*  (at your option). */
+/*   */
+/*  This program is distributed in the hope that it will be useful, */
+/*  but WITHOUT ANY WARRANTY; without even the implied warranty of */
+/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
+/*  GNU General Public License for more details. */
+/*   */
+/*  A copy of the GNU General Public License is available at */
+/*  http://www.r-project.org/Licenses/ */
+
+
 /* vector * matrix */
 
+#ifndef USE_FC_LEN_T
+# define USE_FC_LEN_T
+#endif
 # include <Rinternals.h> /* for length */
 # include <R_ext/Applic.h> /* for dgemm */
+# include <R_ext/Rdynload.h> /* for registering routines */
+#ifndef FCONE
+# define FCONE
+#endif
 
 /* copied from src/main/array.c */
 static void matprod(double *x, int nrx, int ncx,
@@ -31,7 +54,7 @@ static void matprod(double *x, int nrx, int ncx,
 		}
 	} else
 	    F77_CALL(dgemm)(transa, transb, &nrx, &ncy, &ncx, &one,
-			    x, &nrx, y, &nry, &zero, z, &nrx);
+			    x, &nrx, y, &nry, &zero, z, &nrx FCONE FCONE);
     } else /* zero-extent operations should return zeroes */
 	for(i = 0; i < nrx*ncy; i++) z[i] = 0;
 }
@@ -65,22 +88,6 @@ SEXP subprod(SEXP X, SEXP M, SEXP v, SEXP a, SEXP z, SEXP nv) {
   return(X);
 }
 
-/* put results of nonlin localDesignFunction in X */
-
-SEXP nonlin(SEXP X, SEXP a, SEXP z, SEXP expr, SEXP rho) {
-  R_len_t i = INTEGER(a)[0], i1 = 0, last = INTEGER(z)[0];
-  SEXP ans;
-  double *dX, *dans;
-  dX = REAL(X);
-  PROTECT(ans = coerceVector(eval(expr, rho), REALSXP));
-  dans = REAL(ans);
-  for ( ; i <= last;) {
-    dX[i++] = dans[i1++];
-  }
-  UNPROTECT(1);
-  return(X);
-}
-
 /* Computes elementwise products between submatrices of base matrix M and 
    columns of gradient matrix V, summing 'common' results an putting result 
    in submatrix of X. This version has start point in M, V and X for each
@@ -143,3 +150,26 @@ SEXP onecol(SEXP M, SEXP V, SEXP a, SEXP lt, SEXP nr, SEXP nc) {
   UNPROTECT(1);
   return(col);
 }
+
+/* register routines */
+
+static const R_CallMethodDef callMethods[] = {
+    {"submatprod", (DL_FUNC) &submatprod, 5},
+    {"subprod", (DL_FUNC) &subprod, 6},
+    {"newsubprod", (DL_FUNC) &newsubprod, 12},
+    {"onecol", (DL_FUNC) &onecol, 6},
+    {NULL}
+};
+
+void R_init_gnm(DllInfo *info)
+{
+    /* Register the C and .Call routines.
+     No .C(), .Fortran() or .External() routines,
+     so pass those arrays as NULL.
+     */
+    R_registerRoutines(info,
+                       NULL, callMethods,
+                       NULL, NULL);
+    R_useDynamicSymbols(info, FALSE);
+    R_forceSymbols(info, TRUE);
+}
